package com.vance.informit.race;

import junit.framework.Assert;
import org.apache.log4j.AppenderSkeleton;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.spi.LoggingEvent;
import org.junit.Test;

import java.util.NoSuchElementException;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.FutureTask;

public class AuditRaceTest {
    @Test
    public void testPostFlush_EmptyRace()
            throws InterruptedException, ExecutionException {
        // Set up software under test with one record
        AuditRace sut = new AuditRace();
        sut.addRecord();
        // Set up the thread in which to induce the race
        Callable<Void> raceInducer = new PostFlushCallable(sut);
        FutureTask<Void> raceTask = new FutureTask<Void>(raceInducer);
        Thread inducerThread = new Thread(raceTask);
        // Configure log4j for injection
        SynchronizationAppender lock = new SynchronizationAppender();
        Logger log = Logger.getLogger(AuditRace.class);
        log.addAppender(lock);
        log.setLevel(Level.DEBUG);

        inducerThread.start();
        while (Thread.State.WAITING != inducerThread.getState());
        // We don't want this failure to look like the race failure
        try {
            sut.getLogRecords().remove(0);
        } catch (NoSuchElementException nsee) {
            Assert.fail();
        }
        synchronized (lock) {
            lock.notifyAll();
        }

        raceTask.get();
    }

    private class PostFlushCallable implements Callable<Void> {
        private AuditRace race;

        public PostFlushCallable(AuditRace race) {
            this.race = race;
        }

        @Override
        public Void call() {
            race.postFlush();
            return null;
        }
    }

    private class SynchronizationAppender extends AppenderSkeleton {
        @Override
        protected void append(LoggingEvent loggingEvent) {
            try {
                this.wait();
            } catch (InterruptedException e) {
                return;
            }
        }

        @Override
        public void close() {
            // Empty
        }

        @Override
        public boolean requiresLayout() {
            return false;
        }
    }
}
